@@ -13,6 +13,8 @@ where
 import           Prelude ()
 import           Matterhorn.Prelude
 
+import qualified Paths_matterhorn as Paths
+
 import           Control.Monad.Trans.Except
 import           Control.Monad.Trans.Class ( lift )
 import           Data.Char ( isDigit, isAlpha )
@@ -48,7 +50,8 @@ defaultSkylightingPaths :: IO [FilePath]
 defaultSkylightingPaths = do
     xdg <- xdgSyntaxDir
     adjacent <- getBundledSyntaxPath
-    return [xdg, adjacent]
+    cabalDataFiles <- Paths.getDataFileName syntaxDirName
+    return [xdg, adjacent, cabalDataFiles]
 
 getBundledSyntaxPath :: IO FilePath
 getBundledSyntaxPath = do
@@ -115,6 +118,11 @@ fromIni = do
       (configShowMessageTimestamps defaultConfig)
     configActivityBell <- fieldFlagDef "activityBell"
       (configActivityBell defaultConfig)
+    configTruncateVerbatimBlocksInt <- fieldDefOf "truncateVerbatimBlockHeight" number
+      (maybe 0 id $ configTruncateVerbatimBlocks defaultConfig)
+    let configTruncateVerbatimBlocks = case configTruncateVerbatimBlocksInt of
+            i | i <= 0 -> Nothing
+              | otherwise -> Just i
     configHyperlinkingMode <- fieldFlagDef "hyperlinkURLs"
       (configHyperlinkingMode defaultConfig)
     configPass <- (Just . PasswordCommand <$> field "passcmd") <!>
@@ -255,6 +263,7 @@ defaultConfig =
            , configActivityNotifyCommand       = Nothing
            , configActivityNotifyVersion       = NotifyV1
            , configActivityBell                = False
+           , configTruncateVerbatimBlocks      = Nothing
            , configShowMessageTimestamps       = True
            , configShowBackground              = Disabled
            , configShowMessagePreview          = False
