@@ -4,7 +4,7 @@
 module Matterhorn.Draw.Messages
   ( MessageData(..)
   , renderMessage
-  , nameForUserRef
+  , printableNameForUserRef
   , renderSingleMessage
   , unsafeRenderMessageSelection
   , renderLastMessages
@@ -28,7 +28,7 @@ import qualified Data.Text as T
 import qualified Graphics.Vty as V
 import           Lens.Micro.Platform ( (.~), to )
 import           Network.Mattermost.Lenses ( postEditAtL, postCreateAtL )
-import           Network.Mattermost.Types ( ServerTime(..), userUsername, postId )
+import           Network.Mattermost.Types ( ServerTime(..), UserId, userUsername, userId, postId )
 import           Prelude ()
 import           Matterhorn.Prelude
 
@@ -39,38 +39,113 @@ import           Matterhorn.Types
 import           Matterhorn.Types.RichText
 import           Matterhorn.Types.DirectionalSeq
 
+-- | A bundled structure that includes all the information necessary
+-- to render a given message
+data MessageData =
+    MessageData { mdEditThreshold :: Maybe ServerTime
+                -- ^ If specified, any messages edited before this point
+                -- in time are not indicated as edited.
+                , mdShowOlderEdits :: Bool
+                -- ^ Indicates whether "edited" markers should be shown
+                -- for old messages (i.e., ignore the mdEditThreshold
+                -- value).
+                , mdShowReactions :: Bool
+                -- ^ Whether to render reactions.
+                , mdMessage :: Message
+                -- ^ The message to render.
+                , mdUserName :: Maybe Text
+                -- ^ The username of the message's author, if any. This
+                -- is passed here rather than obtaining from the message
+                -- because we need to do lookups in the ChatState to
+                -- compute this, and we don't pass the ChatState into
+                -- renderMessage.
+                , mdParentMessage :: Maybe Message
+                -- ^ The parent message of this message, if any.
+                , mdParentUserName :: Maybe Text
+                -- ^ The author of the parent message, if any.
+                , mdThreadState :: ThreadState
+                -- ^ The thread state of this message.
+                , mdRenderReplyParent :: Bool
+                -- ^ Whether to render the parent message.
+                , mdHighlightSet :: HighlightSet
+                -- ^ The highlight set to use to highlight usernames,
+                -- channel names, etc.
+                , mdIndentBlocks :: Bool
+                -- ^ Whether to indent the message underneath the
+                -- author's name (True) or just display it to the right
+                -- of the author's name (False).
+                , mdTruncateVerbatimBlocks :: Maybe Int
+                -- ^ At what height to truncate long verbatim/code blocks.
+                , mdMessageWidthLimit :: Maybe Int
+                -- ^ A width override to use to wrap non-code blocks
+                -- and code blocks without syntax highlighting. If
+                -- unspecified, all blocks in the message will be
+                -- wrapped and truncated at the width specified by the
+                -- rendering context. If specified, all non-code blocks
+                -- will be wrapped at this width and highlighted code
+                -- blocks will be rendered using the context's width.
+                , mdMyUsername :: Text
+                -- ^ The username of the user running Matterhorn.
+                , mdMyUserId :: UserId
+                -- ^ The user ID of the user running Matterhorn.
+                , mdWrapNonhighlightedCodeBlocks :: Bool
+                -- ^ Whether to wrap text in non-highlighted code
+                -- blocks.
+                }
+
 maxMessageHeight :: Int
 maxMessageHeight = 200
 
--- | nameForUserRef converts the UserRef into a printable name, based
--- on the current known user data.
-nameForUserRef :: ChatState -> UserRef -> Maybe Text
-nameForUserRef st uref =
+botUserLabel :: T.Text
+botUserLabel = "[BOT]"
+
+pinIndicator :: T.Text
+pinIndicator = "[PIN]"
+
+-- | printableNameForUserRef converts the UserRef into a printable name,
+-- based on the current known user data.
+printableNameForUserRef :: ChatState -> UserRef -> Maybe Text
+printableNameForUserRef st uref =
     case uref of
         NoUser -> Nothing
         UserOverride _ t -> Just t
         UserI _ uId -> displayNameForUserId uId st
 
 -- | renderSingleMessage is the main message drawing function.
---
--- The `ind` argument specifies an "indicator boundary".  Showing
--- various indicators (e.g. "edited") is not typically done for
--- messages that are older than this boundary value.
 renderSingleMessage :: ChatState
+                    -- ^ The application state
                     -> HighlightSet
+                    -- ^ The highlight set to use when rendering this
+                    -- message
                     -> Maybe ServerTime
+                    -- ^ This specifies an "indicator boundary". Showing
+                    -- various indicators (e.g. "edited") is not
+                    -- typically done for messages that are older than
+                    -- this boundary value.
                     -> Message
+                    -- ^ The message to render
                     -> ThreadState
+                    -- ^ The thread state in which to render the message
                     -> Widget Name
 renderSingleMessage st hs ind m threadState =
   renderChatMessage st hs ind threadState (withBrackets . renderTime st . withServerTime) m
 
 renderChatMessage :: ChatState
+                  -- ^ The application state
                   -> HighlightSet
+                  -- ^ The highlight set to use when rendering this
+                  -- message
                   -> Maybe ServerTime
+                  -- ^ This specifies an "indicator boundary". Showing
+                  -- various indicators (e.g. "edited") is not typically
+                  -- done for messages that are older than this boundary
+                  -- value.
                   -> ThreadState
+                  -- ^ The thread state in which to render the message
                   -> (ServerTime -> Widget Name)
+                  -- ^ A function to render server times
                   -> Message
+                  -- ^ The message to render
                   -> Widget Name
 renderChatMessage st hs ind threadState renderTimeFunc msg =
     let showOlderEdits = configShowOlderEdits config
@@ -81,9 +156,9 @@ renderChatMessage st hs ind threadState renderTimeFunc msg =
           InReplyTo pId -> getMessageForPostId st pId
         m = renderMessage MessageData
               { mdMessage           = msg
-              , mdUserName          = msg^.mUser.to (nameForUserRef st)
+              , mdUserName          = msg^.mUser.to (printableNameForUserRef st)
               , mdParentMessage     = parent
-              , mdParentUserName    = parent >>= (^.mUser.to (nameForUserRef st))
+              , mdParentUserName    = parent >>= (^.mUser.to (printableNameForUserRef st))
               , mdEditThreshold     = ind
               , mdHighlightSet      = hs
               , mdShowOlderEdits    = showOlderEdits
@@ -93,7 +168,9 @@ renderChatMessage st hs ind threadState renderTimeFunc msg =
               , mdShowReactions     = True
               , mdMessageWidthLimit = Nothing
               , mdMyUsername        = userUsername $ myUser st
+              , mdMyUserId          = userId $ myUser st
               , mdWrapNonhighlightedCodeBlocks = True
+              , mdTruncateVerbatimBlocks = st^.csVerbatimTruncateSetting
               }
         fullMsg =
           case msg^.mUser of
@@ -131,7 +208,11 @@ unsafeRenderMessageSelection :: (SeqDirection dir1, SeqDirection dir2)
                                   , DirectionalSeq dir2 (Message, ThreadState)
                                   )
                                 )
+                             -- ^ The message to render, the messages
+                             -- before it, and after it, respectively
                              -> (Message -> ThreadState -> Widget Name)
+                             -- ^ A per-message rendering function to
+                             -- use
                              -> Widget Name
 unsafeRenderMessageSelection ((curMsg, curThreadState), (before, after)) doMsgRender =
   Widget Greedy Greedy $ do
@@ -173,9 +254,6 @@ unsafeRenderMessageSelection ((curMsg, curThreadState), (before, after)) doMsgRe
                          then cropBottom (lowerHeight - curHeight) lowerHalf
                          else cropBottom lowerHeight lowerHalf)
 
-resultToWidget :: Result n -> Widget n
-resultToWidget = Widget Fixed Fixed . return
-
 renderMessageSeq :: (SeqDirection dir)
                  => Int
                  -> (Message -> ThreadState -> Widget Name)
@@ -242,7 +320,7 @@ renderLastMessages st hs editCutoff msgs =
                     return $ result : rest
 
         results <- go targetHeight msgs
-        render $ vBox $ (Widget Fixed Fixed . return) <$> reverse results
+        render $ vBox $ resultToWidget <$> reverse results
 
 relaxHeight :: Context -> Context
 relaxHeight c = c & availHeightL .~ (max maxMessageHeight (c^.availHeightL))
@@ -259,82 +337,41 @@ render1 doMsgRender msg threadState = case msg^.mDeleted of
                 render $ padRight Max $
                 doMsgRender msg threadState
 
--- | A bundled structure that includes all the information necessary
--- to render a given message
-data MessageData =
-    MessageData { mdEditThreshold :: Maybe ServerTime
-                -- ^ If specified, any messages edited before this point
-                -- in time are not indicated as edited.
-                , mdShowOlderEdits :: Bool
-                -- ^ Indicates whether "edited" markers should be shown
-                -- for old messages (i.e., ignore the mdEditThreshold
-                -- value).
-                , mdShowReactions :: Bool
-                -- ^ Whether to render reactions.
-                , mdMessage :: Message
-                -- ^ The message to render.
-                , mdUserName :: Maybe Text
-                -- ^ The username of the message's author, if any. This
-                -- is passed here rather than obtaining from the message
-                -- because we need to do lookups in the ChatState to
-                -- compute this, and we don't pass the ChatState into
-                -- renderMessage.
-                , mdParentMessage :: Maybe Message
-                -- ^ The parent message of this message, if any.
-                , mdParentUserName :: Maybe Text
-                -- ^ The author of the parent message, if any.
-                , mdThreadState :: ThreadState
-                -- ^ The thread state of this message.
-                , mdRenderReplyParent :: Bool
-                -- ^ Whether to render the parent message.
-                , mdHighlightSet :: HighlightSet
-                -- ^ The highlight set to use to highlight usernames,
-                -- channel names, etc.
-                , mdIndentBlocks :: Bool
-                -- ^ Whether to indent the message underneath the
-                -- author's name (True) or just display it to the right
-                -- of the author's name (False).
-                , mdMessageWidthLimit :: Maybe Int
-                -- ^ A width override to use to wrap non-code blocks
-                -- and code blocks without syntax highlighting. If
-                -- unspecified, all blocks in the message will be
-                -- wrapped and truncated at the width specified by the
-                -- rendering context. If specified, all non-code blocks
-                -- will be wrapped at this width and highlighted code
-                -- blocks will be rendered using the context's width.
-                , mdMyUsername :: Text
-                -- ^ The username of the user running Matterhorn.
-                , mdWrapNonhighlightedCodeBlocks :: Bool
-                -- ^ Whether to wrap text in non-highlighted code
-                -- blocks.
-                }
-
--- | renderMessage performs markdown rendering of the specified message.
+-- | This performs rendering of the specified message according to
+-- settings in MessageData.
 renderMessage :: MessageData -> Widget Name
 renderMessage md@MessageData { mdMessage = msg, .. } =
     let msgUsr = case mdUserName of
-          Just u -> if omittedUsernameType (msg^.mType) then Nothing else Just u
+          Just u -> if omittedUsernameType (msg^.mType)
+                    then Nothing
+                    else Just u
           Nothing -> Nothing
-        botElem = if isBotMessage msg then txt "[BOT]" else emptyWidget
+
+        botAuthorElem = if isBotMessage msg
+                        then txt botUserLabel
+                        else emptyWidget
+
         mId = msg^.mMessageId
+
         clickableAuthor un = case mId of
             Nothing -> id
             -- We use the index (-1) since indexes for clickable
             -- usernames elsewhere in this message start at 0.
             Just i -> clickable (ClickableUsernameInMessage i (-1) un)
+
         nameElems = case msgUsr of
           Just un
             | isEmote msg ->
-                [ withDefAttr pinnedMessageIndicatorAttr $ txt $ if msg^.mPinned then "[PIN]" else ""
+                [ withDefAttr pinnedMessageIndicatorAttr $ txt $ if msg^.mPinned then pinIndicator else ""
                 , txt $ (if msg^.mFlagged then "[!] " else "") <> "*"
                 , clickableAuthor un $ colorUsername mdMyUsername un un
-                , botElem
+                , botAuthorElem
                 , txt " "
                 ]
             | otherwise ->
-                [ withDefAttr pinnedMessageIndicatorAttr $ txt $ if msg^.mPinned then "[PIN] " else ""
+                [ withDefAttr pinnedMessageIndicatorAttr $ txt $ if msg^.mPinned then pinIndicator <> " " else ""
                 , clickableAuthor un $ colorUsername mdMyUsername un un
-                , botElem
+                , botAuthorElem
                 , txt $ (if msg^.mFlagged then "[!]" else "") <> ": "
                 ]
           Nothing -> []
@@ -355,58 +392,31 @@ renderMessage md@MessageData { mdMessage = msg, .. } =
 
         augmentedText = unBlocks $ maybeAugment $ msg^.mText
         msgWidget =
-            vBox $ (layout mdHighlightSet mdMessageWidthLimit nameElems augmentedText . viewl) augmentedText :
-                   catMaybes [msgAtch, msgReac]
+            vBox $ (renderBlocks mdHighlightSet mdMessageWidthLimit nameElems augmentedText . viewl) augmentedText :
+                   catMaybes [msgAtch, messageReactions md]
+
         replyIndent = Widget Fixed Fixed $ do
             ctx <- getContext
             -- NB: The amount subtracted here must be the total padding
             -- added below (pad 1 + vBorder)
             w <- render $ hLimit (ctx^.availWidthL - 2) msgWidget
             render $ vLimit (V.imageHeight $ w^.imageL) $
-                padRight (Pad 1) vBorder <+> (Widget Fixed Fixed $ return w)
+                padRight (Pad 1) vBorder <+> resultToWidget w
+
         msgAtch = if S.null (msg^.mAttachments)
           then Nothing
           else Just $ withDefAttr clientMessageAttr $ vBox
-                 [ txt ("  [attached: `" <> a^.attachmentName <> "`]")
+                 [ padLeft (Pad 2) $ clickable (ClickableAttachment (a^.attachmentFileId)) $
+                                     txt ("[attached: `" <> a^.attachmentName <> "`]")
                  | a <- toList (msg^.mAttachments)
                  ]
-        msgReac = if Map.null (msg^.mReactions) || (not mdShowReactions)
-          then Nothing
-          else let renderR e us lst =
-                       let n = Set.size us
-                       in if | n == 1    -> makeReactionWidget e us (" [" <> e <> "]") : lst
-                             | otherwise -> makeReactionWidget e us (" [" <> e <> " " <> T.pack (show n) <> "]") : lst
-                   nonEmptyReactions = Map.filter (not . Set.null) $ msg^.mReactions
-                   makeReactionWidget e us t =
-                       let w = txt t in
-                       maybe w (flip clickable w) $ makeName e us
-                   hasAnyReactions = not $ null nonEmptyReactions
-                   makeName e us = do
-                       pid <- postId <$> msg^.mOriginalPost
-                       Just $ ClickableReactionInMessage pid e us
-                   reactionWidget = Widget Fixed Fixed $ do
-                       ctx <- getContext
-                       let lineW = ctx^.availWidthL
-                       reacs <- mapM render $ Map.foldrWithKey renderR [] nonEmptyReactions
-                       let reacLines :: [Result n] -> Int -> [Result n] -> [[Result n]]
-                           reacLines l _ []     = if null l then [] else [l]
-                           reacLines l w (r:rs) =
-                               let rW = V.imageWidth $ r^.imageL
-                               in if rW <= w
-                                  then reacLines (l <> [r]) (w - rW) rs
-                                  else
-                                      let rest = reacLines [] lineW rs
-                                      in l : [r] : rest
 
-                       render $ vBox $ hBox <$> (fmap (fmap resultToWidget)) (reacLines [] lineW reacs)
-               in if hasAnyReactions
-                  then Just $ withDefAttr emojiAttr $ txt "   " <+> reactionWidget
-                  else Nothing
         withParent p =
             case mdThreadState of
-                NoThread -> msgWidget
+                NoThread           -> msgWidget
                 InThreadShowParent -> p <=> replyIndent
-                InThread -> replyIndent
+                InThread           -> replyIndent
+
     in if not mdRenderReplyParent
        then msgWidget
        else case msg^.mInReplyToMsg of
@@ -426,22 +436,24 @@ renderMessage md@MessageData { mdMessage = msg, .. } =
                       in withParent (addEllipsis $ forceAttr replyParentAttr parentMsg)
 
     where
-        layout :: HighlightSet -> Maybe Int -> [Widget Name] -> Seq Block
-               -> ViewL Block -> Widget Name
-        layout hs w nameElems bs xs | length xs > 1     = multiLnLayout hs w nameElems bs
-        layout hs w nameElems bs (Blockquote {} :< _) = multiLnLayout hs w nameElems bs
-        layout hs w nameElems bs (CodeBlock {} :< _)  = multiLnLayout hs w nameElems bs
-        layout hs w nameElems bs (HTMLBlock {} :< _)  = multiLnLayout hs w nameElems bs
-        layout hs w nameElems bs (List {} :< _)       = multiLnLayout hs w nameElems bs
-        layout hs w nameElems bs (Para inlns :< _)
-            | F.any breakCheck (unInlines inlns)      = multiLnLayout hs w nameElems bs
-        layout hs w nameElems bs _                    = nameNextToMessage hs w nameElems bs
+        renderBlocks :: HighlightSet -> Maybe Int -> [Widget Name] -> Seq Block
+                     -> ViewL Block -> Widget Name
+        renderBlocks hs w nameElems bs xs | length xs > 1   = multiLnLayout hs w nameElems bs
+        renderBlocks hs w nameElems bs (Blockquote {} :< _) = multiLnLayout hs w nameElems bs
+        renderBlocks hs w nameElems bs (CodeBlock {} :< _)  = multiLnLayout hs w nameElems bs
+        renderBlocks hs w nameElems bs (HTMLBlock {} :< _)  = multiLnLayout hs w nameElems bs
+        renderBlocks hs w nameElems bs (List {} :< _)       = multiLnLayout hs w nameElems bs
+        renderBlocks hs w nameElems bs (Para inlns :< _)
+            | F.any isBreak (unInlines inlns)               = multiLnLayout hs w nameElems bs
+        renderBlocks hs w nameElems bs _                    = nameNextToMessage hs w nameElems bs
 
         multiLnLayout hs w nameElems bs =
             if mdIndentBlocks
                then vBox [ hBox nameElems
                          , hBox [txt "  ", renderRichText mdMyUsername hs ((subtract 2) <$> w)
-                                                 mdWrapNonhighlightedCodeBlocks (Just clickableNames) (Blocks bs)]
+                                                 mdWrapNonhighlightedCodeBlocks
+                                                 mdTruncateVerbatimBlocks
+                                                 (Just mkClickableNames) (Blocks bs)]
                          ]
                else nameNextToMessage hs w nameElems bs
 
@@ -449,21 +461,64 @@ renderMessage md@MessageData { mdMessage = msg, .. } =
             Widget Fixed Fixed $ do
                 nameResult <- render $ hBox nameElems
                 let newW = subtract (V.imageWidth (nameResult^.imageL)) <$> w
-                render $ hBox [ Widget Fixed Fixed $ return nameResult
-                              , renderRichText mdMyUsername hs newW mdWrapNonhighlightedCodeBlocks (Just clickableNames) (Blocks bs)
+                render $ hBox [ resultToWidget nameResult
+                              , renderRichText mdMyUsername hs newW
+                                  mdWrapNonhighlightedCodeBlocks
+                                  mdTruncateVerbatimBlocks
+                                  (Just mkClickableNames) (Blocks bs)
                               ]
 
-        breakCheck i = i `elem` [ELineBreak, ESoftBreak]
+        isBreak i = i `elem` [ELineBreak, ESoftBreak]
 
-        clickableNames i (EHyperlink u _) =
+        mkClickableNames i (EHyperlink u _) =
             case msg^.mMessageId of
                 Just mId -> Just $ ClickableURLInMessage mId i $ LinkURL u
                 Nothing -> Nothing
-        clickableNames i (EUser name) =
+        mkClickableNames i (EUser name) =
             case msg^.mMessageId of
                 Just mId -> Just $ ClickableUsernameInMessage mId i name
                 Nothing -> Nothing
-        clickableNames _ _ = Nothing
+        mkClickableNames _ _ = Nothing
+
+messageReactions :: MessageData -> Maybe (Widget Name)
+messageReactions MessageData { mdMessage = msg, .. } =
+    if Map.null (msg^.mReactions) || (not mdShowReactions)
+    then Nothing
+    else let renderR e us lst =
+                 let n = Set.size us
+                     mine = isMyReaction us
+                     content = if | n == 1    -> "[" <> e <> "]"
+                                  | otherwise -> "[" <> e <> " " <> T.pack (show n) <> "]"
+                     w = makeReactionWidget mine e us content
+                 in padRight (Pad 1) w : lst
+             nonEmptyReactions = Map.filter (not . Set.null) $ msg^.mReactions
+             isMyReaction = Set.member mdMyUserId
+             makeReactionWidget mine e us t =
+                 let w = withDefAttr attr $ txt t
+                     attr = if mine then myReactionAttr else reactionAttr
+                 in maybe w (flip clickable w) $ makeName e us
+             hasAnyReactions = not $ null nonEmptyReactions
+             makeName e us = do
+                 pid <- postId <$> msg^.mOriginalPost
+                 Just $ ClickableReactionInMessage pid e us
+             reactionWidget = Widget Fixed Fixed $ do
+                 ctx <- getContext
+                 let lineW = ctx^.availWidthL
+                 reacs <- mapM render $ Map.foldrWithKey renderR [] nonEmptyReactions
+                 let reacLines :: [Result n] -> Int -> [Result n] -> [[Result n]]
+                     reacLines l _ []     = if null l then [] else [l]
+                     reacLines l w (r:rs) =
+                         let rW = V.imageWidth $ r^.imageL
+                         in if rW <= w
+                            then reacLines (l <> [r]) (w - rW) rs
+                            else if rW > lineW
+                                 then l : [r] : reacLines [] lineW rs
+                                 else l : reacLines [] lineW (r:rs)
+
+                 render $ vBox $ hBox <$> (fmap (fmap resultToWidget)) (reacLines [] lineW reacs)
+         in if hasAnyReactions
+            then Just $ txt "   " <+> reactionWidget
+            else Nothing
 
 -- Add the edit sentinel to the end of the last block in the sequence.
 -- If the last block is a paragraph, append it to that paragraph.
@@ -484,17 +539,17 @@ appendEditSentinel sentinel b =
 
 omittedUsernameType :: MessageType -> Bool
 omittedUsernameType = \case
-    CP Join -> True
-    CP Leave -> True
+    CP Join        -> True
+    CP Leave       -> True
     CP TopicChange -> True
-    _ -> False
+    _              -> False
 
 addEllipsis :: Widget a -> Widget a
 addEllipsis w = Widget (hSize w) (vSize w) $ do
     ctx <- getContext
     let aw = ctx^.availWidthL
     result <- render w
-    let withEllipsis = (hLimit (aw - 3) $ vLimit 1 $ (Widget Fixed Fixed $ return result)) <+>
+    let withEllipsis = (hLimit (aw - 3) $ vLimit 1 $ (resultToWidget result)) <+>
                        str "..."
     if (V.imageHeight (result^.imageL) > 1) || (V.imageWidth (result^.imageL) == aw) then
         render withEllipsis else
