@@ -31,32 +31,35 @@ import qualified Data.Text as T
 import qualified Graphics.Vty as V
 import qualified Skylighting.Core as Sky
 
-import           Matterhorn.Constants ( normalChannelSigil, userSigil, editMarking )
+import           Matterhorn.Constants ( normalChannelSigil, editMarking )
 import           Matterhorn.Draw.RichText.Flatten
 import           Matterhorn.Draw.RichText.Wrap
 import           Matterhorn.Themes
-import           Matterhorn.Types ( HighlightSet(..), emptyHSet, SemEq(..) )
+import           Matterhorn.Types ( HighlightSet(..), emptyHSet, SemEq(..)
+                                  , addUserSigil, resultToWidget )
 import           Matterhorn.Types.RichText
 
 
--- Render markdown with username highlighting
+-- | Render rich text.
 renderRichText :: SemEq a
                => Text
                -- ^ The username of the currently-authenticated user.
                -> HighlightSet
                -- ^ A highlight set for highlighting channel and
-               -- usernames.
+               -- usernames and code blocks.
                -> Maybe Int
                -- ^ An optional maximum width.
                -> Bool
                -- ^ Whether to do line wrapping.
+               -> Maybe Int
+               -- ^ At what height to truncate long verbatim/code blocks
                -> Maybe (Int -> Inline -> Maybe a)
                -- ^ An optional function to build resource names for
                -- clickable regions.
                -> Blocks
                -- ^ The content to render.
                -> Widget a
-renderRichText curUser hSet w doWrap nameGen (Blocks bs) =
+renderRichText curUser hSet w doWrap doVerbTrunc nameGen (Blocks bs) =
     runReader (do
               blocks <- mapM renderBlock (addBlankLines bs)
               return $ B.vBox $ toList blocks)
@@ -64,6 +67,7 @@ renderRichText curUser hSet w doWrap nameGen (Blocks bs) =
                        , drawHighlightSet = hSet
                        , drawLineWidth = w
                        , drawDoLineWrapping = doWrap
+                       , drawTruncateVerbatimBlocks = doVerbTrunc
                        , drawNameGen = nameGen
                        })
 
@@ -86,7 +90,7 @@ renderText' :: SemEq a
             -- ^ The text to parse and then render as rich text.
             -> Widget a
 renderText' baseUrl curUser hSet nameGen t =
-    renderRichText curUser hSet Nothing True nameGen $
+    renderRichText curUser hSet Nothing True Nothing nameGen $
         parseMarkdown baseUrl t
 
 -- Add blank lines only between adjacent elements of the same type, to
@@ -121,6 +125,7 @@ data DrawCfg a =
             , drawHighlightSet :: HighlightSet
             , drawLineWidth :: Maybe Int
             , drawDoLineWrapping :: Bool
+            , drawTruncateVerbatimBlocks :: Maybe Int
             , drawNameGen :: Maybe (Int -> Inline -> Maybe a)
             }
 
@@ -162,7 +167,11 @@ renderBlock (CodeBlock ci tx) = do
         mSyntax = do
             lang <- codeBlockLanguage ci
             Sky.lookupSyntax lang (hSyntaxMap hSet)
-    f tx
+    w <- f tx
+    trunc <- asks drawTruncateVerbatimBlocks
+    case trunc of
+        Nothing -> return w
+        Just maxHeight -> return $ maybeTruncVerbatim maxHeight w
 renderBlock (HTMLBlock t) = do
     w <- asks drawLineWidth
     return $ maybeHLimit w $ textWithCursor t
@@ -170,6 +179,18 @@ renderBlock (HRule) = do
     w <- asks drawLineWidth
     return $ maybeHLimit w $ B.vLimit 1 (B.fill '*')
 
+maybeTruncVerbatim :: Int -> B.Widget n -> B.Widget n
+maybeTruncVerbatim maxHeight w =
+    Widget (B.hSize w) (B.vSize w) $ do
+        result <- render w
+        let h = V.imageHeight (result^.B.imageL)
+        if h > maxHeight
+           then render $ B.vBox [ B.vLimit maxHeight $ B.Widget B.Fixed B.Fixed $ return result
+                                , B.withDefAttr verbatimTruncateMessageAttr $
+                                  B.str $ "(Showing " <> show maxHeight <> " of " <> show h <> " lines)"
+                                ]
+           else return result
+
 quoteChar :: Char
 quoteChar = '>'
 
@@ -216,7 +237,7 @@ renderRawCodeBlock tx = do
             let textHeight = V.imageHeight $ renderedText^.imageL
                 padding = B.padLeftRight 1 (B.vLimit textHeight B.vBorder)
 
-            render $ padding <+> (Widget Fixed Fixed $ return renderedText)
+            render $ padding <+> (resultToWidget renderedText)
 
 renderInlines :: SemEq a => Inlines -> M (Widget a) a
 renderInlines es = do
@@ -283,7 +304,7 @@ renderFlattenedValue curUser (SingleInline fi) = addClickable $ addHyperlink $ a
 
         widget = case val of
             FSpace               -> B.txt " "
-            FUser u              -> colorUsername curUser u $ userSigil <> u
+            FUser u              -> colorUsername curUser u $ addUserSigil u
             FChannel c           -> B.withDefAttr channelNameAttr $
                                     B.txt $ normalChannelSigil <> c
             FEmoji em            -> B.withDefAttr emojiAttr $
