@@ -8,6 +8,7 @@ import qualified Graphics.Vty as Vty
 
 import           Matterhorn.Events.Keybindings
 import           Matterhorn.State.UrlSelect
+import           Matterhorn.State.SaveAttachmentWindow
 import           Matterhorn.Types
 
 
@@ -25,6 +26,9 @@ urlSelectKeyHandlers =
          (Vty.EvKey Vty.KEnter []) $
              openSelectedURL
 
+    , mkKb SaveAttachmentEvent "Save the selected attachment"
+        openSaveAttachmentWindow
+
     , mkKb CancelEvent "Cancel URL selection" stopUrlSelect
 
     , mkKb SelectUpEvent "Move cursor up" $
