@@ -6,6 +6,7 @@ module Matterhorn.State.MessageSelect
   , pinSelectedMessage
   , viewSelectedMessage
   , fillSelectedGap
+  , copyPostLink
   , yankSelectedMessageVerbatim
   , yankSelectedMessage
   , openSelectedMessageURLs
@@ -40,7 +41,7 @@ import           Matterhorn.State.Common
 import           Matterhorn.State.Links
 import           Matterhorn.State.Messages
 import           Matterhorn.Types
-import           Matterhorn.Types.RichText ( findVerbatimChunk )
+import           Matterhorn.Types.RichText ( findVerbatimChunk, makePermalink )
 import           Matterhorn.Types.Common
 import           Matterhorn.Windows.ViewMessage
 
@@ -112,6 +113,18 @@ viewSelectedMessage = do
       | not (isGap msg) -> viewMessage msg
     _        -> return ()
 
+copyPostLink :: MH ()
+copyPostLink = do
+  selected <- use (to getSelectedMessage)
+  case selected of
+    Just msg | isPostMessage msg -> do
+        tId <- use csCurrentTeamId
+        baseUrl <- getServerBaseUrl tId
+        let Just pId = messageIdPostId =<< _mMessageId msg
+        copyToClipboard $ makePermalink baseUrl pId
+        setMode Main
+    _ -> return ()
+
 fillSelectedGap :: MH ()
 fillSelectedGap = do
   selected <- use (to getSelectedMessage)
@@ -159,11 +172,7 @@ openSelectedMessageURLs = whenMode MessageSelect $ do
 
     let urls = msgURLs curMsg
     when (not (null urls)) $ do
-        openedAll <- and <$> mapM (openLinkTarget . _linkTarget) urls
-        case openedAll of
-            True -> return ()
-            False ->
-                mhError $ ConfigOptionMissing "urlOpenCommand"
+        mapM_ (openLinkTarget . _linkTarget) urls
 
 beginConfirmDeleteSelectedMessage :: MH ()
 beginConfirmDeleteSelectedMessage = do
